"""Exceptions triggered to warn about internal errors in Assonant Data Sender module."""


class AssonantDataSenderError(Exception):
    """Assonant Data Sender Error.

    Custom Exception to warn when any error occurs inside AssonantDataSender
    """

    pass
