"""Assonant Beam Visualiation System (BVS) data class."""
from .instrument import Instrument


class BVS(Instrument):
    """Data class to handle all data required to define a Beam Visualization System (BVS)."""
